From 07bd10d0d9b0ac404d77dbad9f865aba1a494a6a Mon Sep 17 00:00:00 2001
From: Akash Mehrotra <mehrotra.akash@gmail.com>
Date: Tue, 21 Jun 2011 00:34:34 +0530
Subject: [PATCH] luahttp can access and modify equalizer bands

---
 modules/lua/libs/equalizer.c          |   99 +++++++++++++++++++++++++++++++++
 share/lua/http/requests/README.txt    |    6 ++
 share/lua/http/requests/equalizer.xml |   12 +++-
 3 files changed, 115 insertions(+), 2 deletions(-)

diff --git a/modules/lua/libs/equalizer.c b/modules/lua/libs/equalizer.c
index f414974..fcef6ce 100644
--- a/modules/lua/libs/equalizer.c
+++ b/modules/lua/libs/equalizer.c
@@ -35,6 +35,7 @@
 #include <vlc_common.h>
 #include <vlc_aout.h>
 #include <vlc_input.h>
+#include <vlc_charset.h>
 
 #include <lua.h>        /* Low level lua C API */
 #include <lauxlib.h>    /* Higher level C API */
@@ -51,6 +52,11 @@ static int vlclua_preamp_get( lua_State *L )
     if( p_input )
     {
         aout_instance_t *p_aout = input_GetAout( p_input );
+        if ( !p_aout )
+        {
+            vlc_object_release( p_input );
+            return 0;
+        }
         float preamp = var_GetFloat( p_aout, "equalizer-preamp");
         lua_pushnumber( L, preamp );
 
@@ -70,6 +76,11 @@ static int vlclua_preamp_set( lua_State *L )
     if( p_input )
     {
         aout_instance_t *p_aout = input_GetAout( p_input );
+        if ( !p_aout )
+        {
+            vlc_object_release( p_input );
+            return 0;
+        }
         float preamp = luaL_checknumber( L, 1 );
         var_SetFloat( p_aout, "equalizer-preamp",preamp);
         lua_pushnumber( L, preamp );
@@ -81,9 +92,97 @@ static int vlclua_preamp_set( lua_State *L )
     return 0;
 }
 
+/*****************************************************************************
+Bands:
+Band 0:  60 Hz
+Band 1: 170 Hz
+Band 2: 310 Hz
+Band 3: 600 Hz
+Band 4:  1 kHz
+Band 5:  3 kHz
+Band 6:  6 kHz
+Band 7: 12 kHz
+Band 8: 14 kHz
+Band 9: 16 kHz
+*****************************************************************************/
+/*****************************************************************************
+* Get the equalizer level for the specified band
+*****************************************************************************/
+static int vlclua_equalizer_get( lua_State *L )
+{
+    input_thread_t *p_input = vlclua_get_input_internal( L );
+    if( p_input )
+    {
+        float level = 0 ;
+        aout_instance_t *p_aout = input_GetAout( p_input );
+        if ( !p_aout )
+        {
+            vlc_object_release( p_input );
+            return 0;
+        }
+        int bandid = luaL_checknumber( L, 1 );
+        char *bands = var_GetString( p_aout, "equalizer-bands" );
+        char *tmp;
+        while( bandid >= 0 )
+        {
+            level = us_strtof( bands, &tmp);
+            bandid--;
+            bands = tmp;
+        }
+        lua_pushnumber( L, level );
+
+        vlc_object_release( p_aout );
+        vlc_object_release( p_input );
+        return 1;
+    }
+    return 0;
+}
+/*****************************************************************************
+* Set the equalizer level for the specified band
+*****************************************************************************/
+static int vlclua_equalizer_set( lua_State *L )
+{
+    input_thread_t *p_input = vlclua_get_input_internal( L );
+    if( p_input )
+    {
+        int i_pos = 0 , j = 0;
+        aout_instance_t *p_aout = input_GetAout( p_input );
+        if ( !p_aout )
+        {
+            vlc_object_release( p_input );
+            return 0;
+        }
+        int bandid = luaL_checknumber( L, 1 );
+        float level = luaL_checknumber( L, 2 );
+        char *bands = var_GetString( p_aout, "equalizer-bands" );
+        char newstr[7];
+        while( j != bandid )
+        {
+            i_pos++;
+            if( bands[i_pos] == '.' )
+            {
+                i_pos++;
+                j++;
+            }
+        }
+        if( bandid != 0 )
+            i_pos++;
+        snprintf( newstr, 7, "%6.1f", level);
+        for( int i = 0 ; i < 6 ; i++ )
+            bands[i_pos+i] = newstr[i];
+        var_SetString( p_aout, "equalizer-bands",bands );
+
+        vlc_object_release( p_aout );
+        vlc_object_release( p_input );
+        return 1;
+    }
+    return 0;
+}
 static const luaL_Reg vlclua_equalizer_reg[] = {
     { "preampget", vlclua_preamp_get },
     { "preampset", vlclua_preamp_set },
+    { "equalizerget", vlclua_equalizer_get },
+    { "equalizerset", vlclua_equalizer_set },
     { NULL, NULL }
 };
 
diff --git a/share/lua/http/requests/README.txt b/share/lua/http/requests/README.txt
index a327ff2..a367adc 100644
--- a/share/lua/http/requests/README.txt
+++ b/share/lua/http/requests/README.txt
@@ -129,3 +129,9 @@ equalizer.xml:
 =============
 >command=preamp&val=<val in dB> 
  sets the preamp value, must be >=-20 and <=20
+
+>command=equalizer&band=<band>&val=<gain in dB, must be >=-20 and <=20)
+
+<Displays the equalizer band gains.
+Band 0: 60 Hz, 1: 170 Hz, 2: 310 Hz, 3: 600 Hz, 4: 1 kHz, 
+5: 3 kHz, 6: 6 kHz, 7: 12 kHz , 8: 14 kHz , 9: 16 kHz
diff --git a/share/lua/http/requests/equalizer.xml b/share/lua/http/requests/equalizer.xml
index 7476487..d71c6ff 100644
--- a/share/lua/http/requests/equalizer.xml
+++ b/share/lua/http/requests/equalizer.xml
@@ -28,11 +28,19 @@ vim:syntax=lua
 
 local command = _GET['command']
 local val = _GET['val']
+local band = _GET['band']
 function round(what, precision)
-   return math.floor(what*math.pow(10,precision)+0.5) / math.pow(10,precision)
+if what then return math.floor(what*math.pow(10,precision)+0.5) / math.pow(10,precision) else return "" end
 end
-if command == "preamp" then vlc.equalizer.preampset(val) end
+if command == "preamp" then vlc.equalizer.preampset(val) 
+elseif command == "equalizer" then vlc.equalizer.equalizerset(band,val)
+end
+freq = { 60 , 170 , 310 , 600 , 1000 , 3000 , 6000 , 12000 , 14000 , 16000 }
 ?>
 <root>
   <preamp><?vlc print(round(vlc.equalizer.preampget(),2)) ?></preamp>
+  <equalizer>
+    <?vlc for i = 0,9 
+    do print("<band id='"..i.."' freqency = '"..freq[i+1].."'>"..round(vlc.equalizer.equalizerget(i),1).."</band>") end ?>
+  </equalizer>
 </root>
-- 
1.7.4.1

